from dataclasses import dataclass

from cal_tools.calcat_interface2 import ShimadzuHPVX2Conditions


class ShimadzuHPVX2:
    image_key = "data.image.pixels"
    copy_keys = [
        "data.image.binning",
        "data.image.dimTypes",
        "data.image.dims",
        "data.image.flipX",
        "data.image.flipY",
        "data.image.roiOffsets",
        "data.image.rotation",
    ]

    def __init__(self, source_name_pattern: str, channel=None, image_key=None):
        self.source_name_pattern = source_name_pattern
        if channel is not None:
            self.channel = channel
        if image_key is not None:
            self.image_key = image_key
        self.image_index_group = self.image_key.partition('.')[0]
        self.instrument = source_name_pattern.split('_')[0]

    def conditions(self, dc: "DataCollection", module=None):  # noqa: F821
        if module is None:
            source_pattern = self.source_name_pattern.format(modno='*')
            det_dc = dc.select(source_pattern)
            if not det_dc.instrument_sources:
                raise ValueError("No detector sources are found")

            source_name = list(det_dc.instrument_sources)[0]
        else:
            source_name = self.instrument_source(module)
        keydata = dc[source_name, self.image_key]
        num_frames = keydata.shape[-3]
        return ShimadzuHPVX2Conditions(burst_frame_count=float(num_frames))

    def instrument_source(self, module: int):
        return self.source_name_pattern.format(modno=module)

    def corrected_source(self, module: int):
        source_name = self.source_name_pattern.format(modno=module)

        # Replace type with CORR.
        parts = source_name.split('/')
        parts[1] = "CORR"
        source_name = '/'.join(parts)

        # Replace channel with output.
        source_name = source_name[:source_name.index(':')] + ':output'

        return source_name
